@@ -78,7 +78,7 @@ function collectLibraryDependencies(product, isDarwin) {
     var publicDeps = {};
     var objects = [];
     var objectByFilePath = {};
-    var tagForLinkingAgainstSharedLib = product.qbs.toolchain.contains("mingw")
+    var tagForLinkingAgainstSharedLib = product.cpp.imageFormat === "pe"
             ? "dynamiclibrary_import" : "dynamiclibrary";
 
     function addObject(obj, addFunc) {
@@ -278,12 +278,11 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
         }
 
         if (isDarwin) {
-            var internalVersion = product.cpp.internalVersion;
-            if (internalVersion && isNumericProductVersion(internalVersion))
-                args.push("-current_version", internalVersion);
+            if (product.cpp.internalVersion)
+                args.push("-current_version", product.cpp.internalVersion);
             escapableLinkerFlags.push("-install_name", UnixUtils.soname(product,
                                                                         primaryOutput.fileName));
-        } else {
+        } else if (product.cpp.imageFormat === "elf") {
             escapableLinkerFlags.push("-soname=" + UnixUtils.soname(product,
                                                                     primaryOutput.fileName));
         }
@@ -295,7 +294,7 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
     if (primaryOutput.fileTags.containsAny(["dynamiclibrary", "loadablemodule"])) {
         if (isDarwin)
             escapableLinkerFlags.push("-headerpad_max_install_names");
-        else
+        else if (product.cpp.imageFormat === "elf")
             escapableLinkerFlags.push("--as-needed");
     }
 
@@ -344,13 +343,15 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
         return rpath;
     }
 
-    function isNotSystemRunPath(p) {
-        return !FileInfo.isAbsolutePath(p) || (!systemRunPaths.contains(p)
-                && !canonicalSystemRunPaths.contains(File.canonicalFilePath(p)));
-    };
-    for (i in rpaths) {
-        if (isNotSystemRunPath(rpaths[i]))
-            escapableLinkerFlags.push("-rpath", fixupRPath(rpaths[i]));
+    if (!product.qbs.targetOS.contains("windows")) {
+        function isNotSystemRunPath(p) {
+            return !FileInfo.isAbsolutePath(p) || (!systemRunPaths.contains(p)
+                    && !canonicalSystemRunPaths.contains(File.canonicalFilePath(p)));
+        };
+        for (i in rpaths) {
+            if (isNotSystemRunPath(rpaths[i]))
+                escapableLinkerFlags.push("-rpath", fixupRPath(rpaths[i]));
+        }
     }
 
     if (product.cpp.entryPoint)
@@ -440,9 +441,13 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
     }
 
     var wholeArchiveActive = false;
+    var prevLib;
     for (i = 0; i < libraryDependencies.libraries.length; ++i) {
         var dep = libraryDependencies.libraries[i];
         var lib = dep.filePath;
+        if (lib === prevLib)
+            continue;
+        prevLib = lib;
         if (dep.wholeArchive && !wholeArchiveActive) {
             var wholeArchiveFlag;
             if (isDarwin) {
@@ -509,6 +514,13 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
     if (importLibs)
         escapableLinkerFlags.push("--out-implib", importLibs[0].filePath);
 
+    if (outputs.application && product.cpp.generateLinkerMapFile) {
+        if (isDarwin)
+            escapableLinkerFlags.push("-map", outputs.mem_map[0].filePath);
+        else
+            escapableLinkerFlags.push("-Map=" + outputs.mem_map[0].filePath);
+    }
+
     var escapedLinkerFlags = escapeLinkerFlags(product, inputs, escapableLinkerFlags);
     Array.prototype.push.apply(escapedLinkerFlags, args);
     var driverLinkerFlags = useCompilerDriver ? product.cpp.driverLinkerFlags : undefined;
@@ -850,7 +862,7 @@ function compilerFlags(project, product, input, output, explicitlyDependsOn) {
     }
 
     var positionIndependentCode = input.cpp.positionIndependentCode;
-    if (positionIndependentCode && !product.qbs.toolchain.contains("mingw"))
+    if (positionIndependentCode && !product.qbs.targetOS.contains("windows"))
         args.push('-fPIC');
 
     var cppFlags = input.cpp.cppFlags;
@@ -1031,7 +1043,7 @@ function linkerEnvVars(config, inputs)
 
 function setResponseFileThreshold(command, product)
 {
-    if (product.qbs.toolchain.contains("mingw") && product.qbs.hostOS.contains("windows"))
+    if (product.qbs.targetOS.contains("windows") && product.qbs.hostOS.contains("windows"))
         command.responseFileThreshold = 10000;
 }
 
@@ -1428,10 +1440,6 @@ function debugInfoArtifacts(product, variants, debugInfoTagSuffix) {
     return artifacts;
 }
 
-function isNumericProductVersion(version) {
-    return version && version.match(/^([0-9]+\.){0,3}[0-9]+$/);
-}
-
 function dumpMacros(env, compilerFilePath, args, nullDevice, tag) {
     var p = new Process();
     try {
